<?php

namespace Was\Smartcash\Controller\Payment;

use Magento\Directory\Helper\Data as DirectoryHelperData;
use Magento\Checkout\Model\Type\Onepage;
use Magento\Framework\App\Action\Context;
use Magento\Framework\Controller\ResultFactory;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Model\OrderRepository;
use Magento\Checkout\Model\Session;
use Magento\Checkout\Helper\Data as CheckoutHelper;
use Magento\Quote\Model\Quote;
use Magento\Customer\Model\Group;

class PlaceOrder extends \Magento\Framework\App\Action\Action
{
	const CODE = 'smartcash';
	/**
	 * @var \Magento\Checkout\Model\Session
	 */
	private $_checkoutSession;

	/**
	 * @var \Magento\Quote\Model\QuoteManagement $quoteManagement
	 */
	private $_quoteManagement;

	/**
	 * @var \Magento\Framework\View\Result\PageFactory
	 */
	private $_resultPageFactory;

	/**
	 * @var \Magento\Quote\Api\CartRepositoryInterface
	 */
	private $_quoteRepository;

	/**
	 * @var OrderRepository
	 */
	private $_orderRepository;

	/**
	 * @var \Magento\Store\Model\StoreManagerInterface
	 */
	private $_storeManager;

	/**
	 * @var \Magento\Framework\Event\ManagerInterface
	 */
	protected $_eventManager;

	/**
	 * @var \Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface
	 */
	protected $_transactionBuilder;

	/**
	 * @var \Clikpak\Aps\Model\TransactionFactory
	 */
	protected $_transactionFactory;

	/**
	 * @var \Psr\Log\LoggerInterface
	 */
	protected $_logger;

	/**
	 * @var \Magento\Customer\Model\Customer
	 */
	protected $_customerSession;

	protected $_helper;
	/**
	 * @var CheckoutHelper $checkoutHelper
	 */
	protected $_checkoutHelper;

	protected $objectManager;
	protected $messageManager;
	protected $quoteFactory;
	protected $resultRedirectFactory;
	/**
	 * 
	 * @param Context $context
	 * @param Session $checkoutSession
	 * @param \Magento\Quote\Model\QuoteManagement $quoteManagement
	 * @param \Magento\Framework\View\Result\PageFactory $resultPageFactory
	 * @param \Magento\Quote\Api\CartRepositoryInterface $quoteRepository
	 * @param OrderRepositoryInterface $orderRepository
	 * @param \Magento\Store\Model\StoreManagerInterface $storeManager
	 * @param \Magento\Framework\Event\ManagerInterface $eventManager
	 * @param \Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface $transportBuilder
	 * @param \Clikpak\Aps\Model\TransactionFactory $transactionFactory
	 * @param \Psr\Log\LoggerInterface $logger
	 * @param \Magento\Customer\Model\Session $customerSession
	 * @param CheckoutHelper $checkoutHelper
	 */
	public function __construct(
		Context $context,
		Session $checkoutSession,
		\Magento\Quote\Model\QuoteManagement $quoteManagement,
		\Magento\Framework\View\Result\PageFactory $resultPageFactory,
		\Magento\Quote\Api\CartRepositoryInterface $quoteRepository,
		OrderRepositoryInterface $orderRepository,
		\Magento\Store\Model\StoreManagerInterface $storeManager,
		\Magento\Framework\Event\ManagerInterface $eventManager,
		\Magento\Sales\Model\Order\Payment\Transaction\BuilderInterface $transportBuilder,
		\Was\Smartcash\Model\TransactionFactory $transactionFactory,
		\Psr\Log\LoggerInterface $logger,
		\Magento\Customer\Model\Session $customerSession,
		\Was\Smartcash\Helper\Data $helper,
		CheckoutHelper $checkoutHelper,
		\Magento\Framework\ObjectManagerInterface $objectManager,
		\Magento\Framework\Message\ManagerInterface $messageManager,
		\Magento\Quote\Model\QuoteFactory $quoteFactory,
		\Magento\Framework\Controller\Result\RedirectFactory $resultRedirectFactory
	) {
		parent::__construct($context);
		$this->_checkoutSession = $checkoutSession;
		$this->_quoteManagement = $quoteManagement;
		$this->_resultPageFactory = $resultPageFactory;
		$this->_quoteRepository = $quoteRepository;
		$this->_orderRepository = $orderRepository;
		$this->_storeManager = $storeManager;
		$this->_eventManager = $eventManager;
		$this->_transactionBuilder = $transportBuilder;
		$this->_transactionFactory = $transactionFactory;
		$this->_logger = $logger;
		$this->_customerSession = $customerSession;
		$this->_helper = $helper;
		$this->_checkoutHelper = $checkoutHelper;
		$this->_objectManager = $objectManager;
		$this->messageManager = $messageManager;
		$this->quoteFactory = $quoteFactory;
		$this->resultRedirectFactory = $resultRedirectFactory;
	}

	public function execute()
	{
		$cartUrl = 'checkout/cart';
		$resultRedirect = $this->resultRedirectFactory->create();
		$smartcashResponse = $this->getRequest()->getParams();
		//echo "<pre>"; print_r($smartcashResponse); die('here');
		$merchantUsername = $this->_helper->getUserName();
		$merchantPassword = $this->_helper->getPassword();
		$transactionToken = $smartcashResponse['sc_transaction_token'];
		$request = array(
			"transactionToken" => $transactionToken,
			"merchantUsername" => $merchantUsername,
			"merchantPassword" => $merchantPassword
		);

		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $this->_helper->getTransactionStatusUrl());
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $request);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
		curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
		$response = curl_exec($ch);

		// DEBUG
		#$urlSC = $this->_helper->getTransactionStatusUrl();
		#@file_put_contents('/tmp/magento', '&&& '.$urlSC."\n", FILE_APPEND);
		#@file_put_contents('/tmp/magento', '&&& '.json_encode($request)."\n", FILE_APPEND);
		#if(curl_errno($ch)) {
		#	file_put_contents('/tmp/magento', '& '.json_encode(curl_error($ch))."\n", FILE_APPEND);
		#}
		#@file_put_contents('/tmp/magento', '&&& '.($response)."\n", FILE_APPEND);
		#@file_put_contents('/tmp/magento', '&&& '.json_encode($smartcashResponse)."\n", FILE_APPEND);
		// - DEBUG

		curl_close($ch);

		if ($response) {
			$result = json_decode($response, 1);
			if ($result) {
				if (isset($result['response']) && isset($result['response']['payed'])) {
					if ($result['response']['payed'] == 1) {
						$smartcashResponse['merchantUsername'] = $merchantUsername;
						$smartcashResponse['merchantPassword'] = $merchantPassword;
						$data = $this->generateOrder($smartcashResponse);

						#DEBUG @file_put_contents('/tmp/magento', json_encode($data)."\n", FILE_APPEND);
						if (isset($data['success']) && $data['success'] == 1) {
							$this->messageManager->addSuccess(__($data['message']));
							return $resultRedirect->setPath('checkout/onepage/success', array('_secure' => true));
						} else {
							$this->messageManager->addError(__($data['message']));
							return $resultRedirect->setPath($cartUrl, ['_secure' => true]);
						}
					} else {
						$this->messageManager->addError(__('The payment was not made.'));
						return $resultRedirect->setPath($cartUrl, ['_secure' => true]);
					}
				}
			} else {
				$this->messageManager->addError(__('The service responded with an invalid JSON.'));
				return $resultRedirect->setPath($cartUrl, ['_secure' => true]);
			}
		} else {
			$this->messageManager->addError(__('The service could not be contacted.'));
			return $resultRedirect->setPath($cartUrl, ['_secure' => true]);
		}
	}

	public function generateOrder($smartcashResponse)
	{

		$resultRedirect = $this->resultRedirectFactory->create();
		/** @var array $switchResponse */
		$cartUrl = 'checkout/cart';

		try {
			//$quote = $this->getQuote();
			if (isset($smartcashResponse['sc_cart_id'])) {
				$quoteId 	= (int)$smartcashResponse['sc_order_id'];
				$quote 		= $this->quoteFactory->create()->load($quoteId);
				$orderNumber 	= $smartcashResponse['sc_cart_id'];
				$status 	= $smartcashResponse['sc_transaction_status'];

				$transactionLog = $this->_transactionFactory->create()->getCollection()
					->addFieldToFilter('merchant_transaction_id', $orderNumber)
					//->addFieldToFilter('status', 'Pending')
					->setOrder('entity_id', 'DESC')
					->getFirstItem();

				if (count($transactionLog->getData()) == 0) {
					$this->messageManager->addError(__('Transaction id is not found.'));
					/** @var \Magento\Framework\Controller\Result\Redirect $resultRedirect */
					return $resultRedirect->setPath($cartUrl, ['_secure' => true]);
				}

				$transactionLog->setPaymentOrderId($smartcashResponse['sc_transaction_token']);
				$transactionLog->setAllResponse(json_encode($smartcashResponse));
				$transactionLog->setStatus($status);
				$transactionLog->save();
				$customerEmail = $transactionLog->getCustomerEmail();

				if ($quote->getId() == '') {
					$this->messageManager->addError(__('Your cart is empty.'));
					/** @var \Magento\Framework\Controller\Result\Redirect $resultRedirect */
					return $resultRedirect->setPath($cartUrl, ['_secure' => true]);
				}

				if ($quote->getItemsCount() <= 0) {
					$this->messageManager->addError(__('No items found'));
					/** @var \Magento\Framework\Controller\Result\Redirect $resultRedirect */
					return $resultRedirect->setPath($cartUrl, ['_secure' => true]);
				}

				if ($this->getCheckoutMethod($quote) === Onepage::METHOD_GUEST) {
					$quote->getBillingAddress()->setEmail($customerEmail);
					$this->prepareGuestQuote($quote);
				}

				$quote->reserveOrderId();
				$quote->setPaymentMethod(self::CODE);

				// Set Sales Order Payment
				$quote->getPayment()->importData(['method' => self::CODE]);

				$quote->collectTotals();
				$this->_quoteRepository->save($quote);
				$this->_checkoutSession->setLastSuccessQuoteId($quote->getId());
				$this->_checkoutSession->setLastQuoteId($quote->getId());
				$this->_checkoutSession->clearHelperData();

				$order = $this->_quoteManagement->submit($quote);
				#OLD $order->setState(\Magento\Sales\Model\Order::STATE_PROCESSING, true);
				#OLD $order->setStatus(\Magento\Sales\Model\Order::STATE_PROCESSING);
				$order->setState(\Magento\Sales\Model\Order::STATE_COMPLETE, true);
				$order->setStatus(\Magento\Sales\Model\Order::STATE_COMPLETE);
				$order->setPaymentStatus('paid');
				#DEBUG @file_put_contents('/tmp/magento', "&&& PAID\n", FILE_APPEND);

				$transactionLog->setOrderId($order->getIncrementId());
				$transactionLog->save();

				$payment = $order->getPayment();
				$payment->setLastTransId($orderNumber);
				$payment->setTransactionId($orderNumber);
				$transactionData = [
					'transaction_id' => $orderNumber,
					'transaction_time' => null,
					'transaction_status' => "Transaction Successfull"
				];
				$payment->setAdditionalInformation([\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $transactionData]);
				$messageTotal = __('The authorized amount');
				$trans = $this->_transactionBuilder;
				$transaction = $trans->setPayment($payment)
					->setOrder($order)
					->setTransactionId($orderNumber)
					->setAdditionalInformation(
						[\Magento\Sales\Model\Order\Payment\Transaction::RAW_DETAILS => (array) $transactionData]
					)
					->setFailSafe(true)
					->build(\Magento\Sales\Model\Order\Payment\Transaction::TYPE_CAPTURE);
				$payment->addTransactionCommentsToOrder($transaction, $messageTotal);
				$payment->setParentTransactionId(null);
				$payment->save();
				$order->addStatusToHistory($order->getStatus(), 'Gateway has authorized the payment.');
				$order->save();

				$this->_checkoutSession->setLastOrderId($order->getId());
				$this->_checkoutSession->setLastRealOrderId($order->getIncrementId());
				$this->_checkoutSession->setLastOrderStatus($order->getStatus());

				//$this->_eventManager->dispatch('sales_order_place_after', ['order' => $order]);

				$successValidator = $this->_objectManager->get('Magento\Checkout\Model\Session\SuccessValidator');
				if (!$successValidator->isValid()) {
					return $resultRedirect->setPath('checkout/cart');
				}

				$data['message'] = 'Your order has been successfully created!';
				$data['success'] = 1;
			}
		} catch (\Exception $e) {
			$data['message'] = $e->getMessage();
			$data['success'] = 0;
		}
		return $data;
	}

	public function getQuote()
	{
		return $this->_checkoutSession->getQuote();
	}

	/**
	 * Get checkout method
	 *
	 * @param Quote $quote
	 * @return string
	 */
	public function getCheckoutMethod(Quote $quote)
	{
		if ($this->_customerSession->isLoggedIn()) {
			return Onepage::METHOD_CUSTOMER;
		}
		if (!$quote->getCheckoutMethod()) {
			if ($this->_checkoutHelper->isAllowedGuestCheckout($quote)) {
				$quote->setCheckoutMethod(Onepage::METHOD_GUEST);
			} else {
				$quote->setCheckoutMethod(Onepage::METHOD_REGISTER);
			}
		}

		return $quote->getCheckoutMethod();
	}

	/**
	 * Prepare quote for guest checkout order submit
	 *
	 * @param Quote $quote
	 * @return void
	 */
	public function prepareGuestQuote(Quote $quote)
	{
		$quote->setCustomerId(null);
		$quote->setCustomerEmail($quote->getBillingAddress()->getEmail());
		$quote->setCustomerIsGuest(true);
		$quote->setCustomerGroupId(Group::NOT_LOGGED_IN_ID);
	}
}
