<?php

/*
 * @Author Switchpay
 */

namespace Was\Smartcash\Helper;

use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Framework\Encryption\EncryptorInterface;
use Magento\Checkout\Model\Session;

class Data extends AbstractHelper {

    /**
     * @var ScopeConfigInterface
     */
    public $_scopeConfig;

    /**
     * @var StoreManagerInterface
     */
    public $_storeManager;

    /**
     * @var Session
     */
    public $_session;

    /**
     * @var EncryptorInterface
     */
    public $_encryptor;

    /**
     * @var \Magento\Store\Api\Data\StoreInterface $store
     */
    public $_store;

    /**
     * @var \Magento\Framework\Locale\Resolver
     */
    public $_resolver;
    
    /**
     * @var \Magento\Framework\UrlInterface
     */
    protected $_urlBuilder;
	
	protected $_context;
    /**
     * Data constructor.
     * @param \Magento\Framework\App\Helper\Context $context
     * @param StoreManagerInterface $storeManager
     * @param Session $session
     * @param \Magento\Framework\Encryption\EncryptorInterface $encryptor
     * @param \Magento\Store\Api\Data\StoreInterface $store
     * @param \Magento\Framework\Locale\Resolver $resolver
     */
    public function __construct(
	Context $_context,
    \Magento\Framework\App\Helper\Context $context,
    StoreManagerInterface $storeManager,
    Session $session,
    EncryptorInterface $encryptor,
    \Magento\Store\Api\Data\StoreInterface $store,
    \Magento\Framework\Locale\Resolver $resolver,
    \Was\Smartcash\Model\TransactionFactory $transactionFactory
    ) {
        $this->_scopeConfig = $context->getScopeConfig();
        $this->_storeManager = $storeManager;
        $this->_session = $session;
        $this->_encryptor = $encryptor;
        $this->_store = $store;
        $this->_resolver = $resolver;
        $this->_urlBuilder = $context->getUrlBuilder();
        $this->_transactionFactory = $transactionFactory;
        parent::__construct($context);
		$this->_url = $_context->getUrl();
    }

    /**
     * @param string $configPath
     * @return bool
     */
    public function getScopeConfig($configPath) {
        return $this->_scopeConfig->getValue(
                        $configPath, ScopeInterface::SCOPE_STORE
        );
    }
    
    public function buildRequest($quote='') {
        if($quote==''){
            $quote = $this->_session->getQuote();
        }
		
		$quoteid = $quote->getId();
        $amount = $quote->getGrandTotal();
		$shippingAmount = $quote->getShippingAddress()->getShippingAmount();
		$taxAmount = $quote->getShippingAddress()->getBaseTaxAmount();
        //$currency_code = $quote->getCurrency()->getData('quote_currency_code');
        $uniqid = uniqid();

		$itemsArray = array();
		$allItems = $quote->getAllVisibleItems();
		$objectManager =\Magento\Framework\App\ObjectManager::getInstance();
		$helperImport = $objectManager->get('\Magento\Catalog\Helper\Image');
		$productRepository = $objectManager->get('\Magento\Catalog\Api\ProductRepositoryInterface');
		foreach ($allItems as $item) {
		//echo 'Sku: '.$item->getSku().'<br />';
			$productId = $item->getProductId();
			$product = $productRepository->getById($productId);
			$imageUrl = $helperImport->init($product, 'product_page_image_small')
                ->setImageFile($product->getSmallImage()) // image,small_image,thumbnail
                ->resize(150)
                ->getUrl();

			$itemsArray[] = array ('title' => $item->getName() ,'description' => $product->getDescription() ,'quantity' => $item->getQty() ,'image' => $imageUrl ,'amount' => $item->getPrice());
		} 
 		
		$queryBuilder = array (
					's2s' => base64_encode($this->_url->getUrl('smartcash/payment/placeOrder')),
					'redirectTo' => base64_encode($this->_url->getUrl('smartcash/payment/placeOrder')),
					'merchantUsername' => $this->getUserName(),
					'merchantPassword' => $this->getPassword(),
					'jsonCart'  => base64_encode(json_encode(
								array (
								'total' => $amount,
								'shipping' => $shippingAmount,
								'additional' => $taxAmount,
								'cart_id' => $uniqid,
								'order_id' => $quoteid,
								'values' => $itemsArray
								)
							)
					)
				);

        $authUrl = $this->getPaymentUrl()."?".http_build_query($queryBuilder);
		
		$response = file_get_contents($authUrl);
		if($response){
			$result = json_decode($response , 1);
			if ( $result ) {
				$status = $result['response']['transaction_status'];
				if ( $status == 1 ){
					$url = $result['response']['transaction_action_url'];
					$saveData = array(
						'merchant_transaction_id' => $uniqid,
						'payment_order_id' => '',
						'status' => 'Pending',
						'amount' => $amount,
						'customer_email' => $quote->getBillingAddress()->getEmail(),
						'quote_id' => $quote->getId(),
					);
					$this->_transactionFactory->create()->setData($saveData)->save();
					$data = array(
						"success" => true,
						"formUrl" => $url,
						"message" => ''
					);
					//header("Location:" . $url );
				}else{
					$message = $result['response']['transaction_message'];
					$data = array(
						"message" => "The service threw an error:" . $message,
						"success" => false,
						"formUrl" => "",
					);
				}
			}else{
				$data = array(
					"message" => "The service responded with an invalid JSON",
					"success" => false,
					"formUrl" => "",
				);
			}
		}else{
            $data = array(
                "message" => "The service could not be contacted.",
                "success" => false,
                "formUrl" => "",
            );
		}
	
        return $data;
    }
    
    public function isEnable() {
        return $this->getScopeConfig('payment/smartcash/active');
    }
    
    public function getUserName() {
        return $this->getScopeConfig('payment/smartcash/user_name');
    }
    
    public function getPassword() {
        return $this->getScopeConfig('payment/smartcash/password');
    }
    
    public function getMode() {
        return $this->getScopeConfig('payment/smartcash/mode');
    }
    
    public function getPaymentUrl() {
        return $this->getScopeConfig('payment/smartcash/payment_url');
    }
    
	public function getTransactionStatusUrl() {
        return $this->getScopeConfig('payment/smartcash/transaction_status_url');
    }
	
    public function sessionCreate($apiUrl) {
        $curl = curl_init();
        curl_setopt_array($curl, array(
            CURLOPT_URL => $apiUrl,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_ENCODING => "",
            CURLOPT_MAXREDIRS => 10,
            CURLOPT_TIMEOUT => 0,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
            CURLOPT_CUSTOMREQUEST => "POST",
            CURLOPT_POSTFIELDS => '',
            CURLOPT_HTTPHEADER => array(
                "Content-Type: application/json",
            ),
        ));
        $curlResponse = curl_exec($curl);
        curl_close($curl);
        $responseData = json_decode($curlResponse, true);
        $response = [];
        if(isset($responseData['errorMessage'])){
            $response['success'] = false;
            $response['data'] = $responseData;
        }
        else{
            if(isset($responseData['orderId']) && isset($responseData['formUrl'])){
                $response['success'] = true;
                $response['data'] = $responseData;
            }
            
        }
        return $response;
    }
}
